package com.zoinla.flir_plugin

import android.content.ContentResolver
import android.content.Context
import android.net.Uri
import android.webkit.MimeTypeMap
import java.io.*

class FileUtils {
    fun getPathFromUri(context: Context, uri: Uri): String? {
        var file: File? = null
        var inputStream: InputStream? = null
        var outputStream: OutputStream? = null
        var success = false
        try {
            val extension = getImageExtension(context, uri)
            inputStream = context.contentResolver.openInputStream(uri)
            file = File.createTempFile("image_picker", extension, context.cacheDir)
            file.deleteOnExit()
            outputStream = FileOutputStream(file)
            if (inputStream != null) {
                copy(inputStream, outputStream)
                success = true
            }
        } catch (ignored: IOException) {
        } finally {
            try {
                inputStream?.close()
            } catch (ignored: IOException) {
            }
            try {
                outputStream?.close()
            } catch (ignored: IOException) {
                // If closing the output stream fails, we cannot be sure that the
                // target file was written in full. Flushing the stream merely moves
                // the bytes into the OS, not necessarily to the file.
                success = false
            }
        }
        return if (success) file!!.path else null
    }

    companion object {
        /** @return extension of image with dot, or default .jpg if it none.
         */
        private fun getImageExtension(context: Context, uriImage: Uri): String {
            var extension: String? = null
            extension = try {
                val imagePath = uriImage.path
                if (uriImage.scheme == ContentResolver.SCHEME_CONTENT) {
                    val mime = MimeTypeMap.getSingleton()
                    mime.getExtensionFromMimeType(context.contentResolver.getType(uriImage))
                } else {
                    MimeTypeMap.getFileExtensionFromUrl(
                            Uri.fromFile(File(uriImage.path)).toString())
                }
            } catch (e: Exception) {
                null
            }
            if (extension == null || extension.isEmpty()) {
                //default extension for matches the previous behavior of the plugin
                extension = "jpg"
            }
            return ".$extension"
        }

        @Throws(IOException::class)
        private fun copy(`in`: InputStream, out: OutputStream) {
            val buffer = ByteArray(4 * 1024)
            var bytesRead: Int
            while (`in`.read(buffer).also { bytesRead = it } != -1) {
                out.write(buffer, 0, bytesRead)
            }
            out.flush()
        }
    }
}